package com.sendFile.protocol.FTP;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import com.moveFile.Movefile;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

public class FTPProtocol {
  String user;
  String password;
  String host;
  int port;
  String local_fileName;
  String remote_filePath;
  String remote_fileName;
  String archive_path;

  public FTPProtocol(
      String user,
      String password,
      String host,
      int port,
      String local_fileName,
      String remote_filePath,
      String remote_fileName,
      String archive_path) {
    this.user = user;
    this.password = password;
    this.host = host;
    this.port = port;
    this.local_fileName = local_fileName;
    this.remote_filePath = remote_filePath;
    this.remote_fileName = remote_fileName;
    this.archive_path = archive_path;
  }

  public void sendFileFTP() {
    System.out.println("Going to SEND files from remote Server via FTP");
    FTPClient ftpClient = new FTPClient();
    try {
      System.out.println(
          "Creating SSH Session: user@host:port |--->"
              + this.user
              + "@"
              + this.host
              + ":"
              + this.port);
      ftpClient.connect(this.host, this.port);
      ftpClient.login(this.user, this.password);
      ftpClient.enterLocalPassiveMode();

      ftpClient.setFileType(2);

      File firstLocalFile = new File(this.local_fileName);

      String[] fileTokens = this.remote_fileName.split("-");

      System.out.println(
          "Going to create brancheise folder:" + fileTokens[0] + " at " + this.remote_filePath);

      ftpClient.changeWorkingDirectory(this.remote_filePath);
      try {
        ftpClient.makeDirectory(fileTokens[0]);
      } catch (Exception ex) {
        System.out.println("Folder not created: " + ex.getMessage());
      }
      ftpClient.changeWorkingDirectory(fileTokens[0]);

      InputStream inputStream = new FileInputStream(firstLocalFile);

      System.out.println("Start uploading file");
      boolean done = ftpClient.storeFile(fileTokens[1], inputStream);
      inputStream.close();
      if (done) {
        System.out.println("The file is uploaded successfully.");
        Movefile mv = new Movefile();
        mv.movefile(firstLocalFile, this.archive_path);
      }
    } catch (Exception ex) {
      System.out.println("Error: " + ex.getMessage());
      ex.printStackTrace();
    } finally {
      try {
        if (ftpClient.isConnected()) {
          ftpClient.logout();
          ftpClient.disconnect();
        }
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  public void fetchFileFTP(String remote_report_pattern) {
    System.out.println("Going to FETCH files from remote Server via FTP");

    FTPClient ftpClient = new FTPClient();

    try {
      ftpClient.connect(this.host, this.port);
      ftpClient.login(this.user, this.password);
      ftpClient.enterLocalPassiveMode();

      ftpClient.setFileType(2);

      ftpClient.changeWorkingDirectory(this.remote_fileName);
      ftpClient.makeDirectory("archive");
      System.out.println("Current directory is " + ftpClient.printWorkingDirectory());
      FTPFile[] ftpFiles = ftpClient.listFiles();

      if (ftpFiles != null && ftpFiles.length > 0) {
        for (FTPFile file : ftpFiles) {
          if (file.isFile()) {

            System.out.println("File is " + file.getName());

            OutputStream output = new FileOutputStream(this.local_fileName + file.getName());

            ftpClient.retrieveFile(file.getName(), output);

            output.close();

            ftpClient.rename(file.getName(), "archive/" + file.getName());
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      try {
        if (ftpClient.isConnected()) {
          ftpClient.logout();
          ftpClient.disconnect();
        }
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }
}